#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import ConfigParser
from math import ceil
from threading import Event, Thread
import time

_logger = logging.getLogger(__name__)

SAVE_EVERY_SECONDS = 60  # [second]


class OperatingDataManager:
    """!
    Verwaltung aller nichtflüchtigen Betriebsparameter
    """
    def __init__(self, config_manager, error_handler):
        """!
        Konstruktor:
        Initialisierung von Config-Manager, Default-Parser, Zählername und Zählerwert
        @param config_manager: Instanz der Klasse ConfigManager
        @param error_handler: Instanz der Klasse ErrorHandler
        """
        _logger.debug("OperatingDataManager")
        self._name = "OperatingDataManager"
        self._parser_default = config_manager.get_parser()
        self._error_handler = error_handler

        self._hf_counter = 0
        self._counter_operating_seconds = 0
        self._opData = None
        try:
            self._op_data_path = self._parser_default.get('path', 'operating_data')
        except:     # TODO: Exception Type
            self._op_data_path = None

        if self._op_data_path is not None:
            self._load_op_data(self._op_data_path)
        else:
            _logger.debug("Can not load operating data file 'operating_data.gs1'")
            self._error_handler.set_error("F1000")

        self._stopped = Event()
        self._stopped.clear()
        self._first_time_not_running = True
        self._is_running = False
        self._start_time = time.time()

        self._stopped.clear()
        self._thread_ohc = Thread(target=self.process, name=self._name)
        self._thread_ohc.setDaemon(True)
        self._thread_ohc.start()
        _logger.debug("Operating-Hours-Counter start")

    def exit(self):
        """
        Stoppt alle Zähler und beendet den Thread.
        """
        self.stop_counter()
        self._stopped.set()

    def _load_op_data(self, op_data_file):
        """!
        Lese aktuellen Zählerstand aus json-File aus
        @param op_data_file: Pfad zur Datei mit den Betriebsdaten
        """
        if op_data_file is not None:
            self._opData = ConfigParser.ConfigParser()        # ConfigParser für ini-Dateien
            self._opData.read(op_data_file)                   # angegebene Datei lesen

            if self._opData.has_section('heatfilecounter'):
                self._hf_counter = int(self._opData.get('heatfilecounter', 'value'))
            else:
                _logger.error("heat file counter not available")

            if self._opData.has_section('operatinghourcounter'):
                self._counter_operating_seconds = int(self._opData.get('operatinghourcounter', 'value'))
            else:
                _logger.error("operating hour counter not available")

    def update_hf_counter(self):
        """!
        Erhöhe lokalen Zählerstand und schreibe neuen Wert in operating_data.gs1
        """
        if self._op_data_path is not None:
            self._hf_counter += 1
            self._opData.set('heatfilecounter', 'value', self._hf_counter)
            with open(self._op_data_path, 'wb') as opDataFile:
                self._opData.write(opDataFile)
        else:
            _logger.debug("Can not save heat file couter to file 'operating_data.gs1'")
            self._error_handler.set_error("F1000")

    def _load_operating_seconds(self):
        """!
        Sichere Betriebsstundenzähler in operating_data.gs1
        @return: Gesicherter Wert der Betriebsstunden
        """
        retval = 0
        if self._op_data_path is not None:
            self._opData.read(self._op_data_path)
            if self._opData.has_section('operatinghourcounter'):
                retval = int(self._opData.get('operatinghourcounter', 'value'))
            else:
                _logger.error("operating hour counter not available")

        return retval

    def _store_operating_seconds(self, value):
        """!
        Sichere Betriebsstundenzähler in operating_data.gs1
        @param value: Zu sichernden Wert der Betriebsstunden
        """
        if self._op_data_path is not None:
            self._opData.set('operatinghourcounter', 'value', value)
            with open(self._op_data_path, 'wb') as opDataFile:
                self._opData.write(opDataFile)
        else:
            _logger.debug("Can not save operating hours to file 'operating_data.gs1'")
            self._error_handler.set_error("F1000")

    def save_process_data(self, hfname='', stage=0):
        """!
        Sichere aktuellen Prozesszustand in operating_data.gs1
        @param hfname: Name der aktuell ausgeführten Heizdatei
        @param stage: Aktuelle Prozessstufe (Fortschritt)
        """
        if self._op_data_path is not None:
            self._opData.set('processdata', 'hfname', hfname)
            self._opData.set('processdata', 'stage', stage)
            with open(self._op_data_path, 'wb') as opDataFile:
                self._opData.write(opDataFile)
        else:
            _logger.debug("Can not save process data to file 'operating_data.gs1'")
            self._error_handler.set_error("F1000")

    def load_process_data(self):
        """!
        Lade Prozesszustand aus operating_data.gs1
        @return Zuletzt ausgeführtes Heizprofil und Stufe, in der das Profil beendet wurde: [hfname, stage]
        """
        hfname = ''
        stage = 0

        if self._op_data_path is not None:
            hfname = self._opData.get('processdata', 'hfname')
            stage = self._opData.get('processdata', 'stage')
        else:
            _logger.debug("Can not load operating data from file 'operating_data.gs1'")
            self._error_handler.set_error("F1000")

        return hfname, stage

    def get_hf_counter(self):
        """!
        Lese aktuellen, lokalen Zählerstand der ausgeführten Heizprozesse aus
        @return Aktueller Zählerstand der ausgeführten Heizprozesse
        """
        return self._hf_counter

        # --- usage commands ---
    def start_counter(self):
        """!
        Startet den Betriebsstundenzähler.
        """
        self._counter_operating_seconds = self._load_operating_seconds()
        self._is_running = True
        self._start_time = time.time()

    def stop_counter(self):
        """!
        Stoppt den Betriebsstundenzähler.
        """
        if self._is_running:
            self._store_operating_seconds(self.get_operating_seconds())
            self._is_running = False

    def get_operating_seconds(self):
        """!
        Gibt die Betriebsstunden der Heizung zurück.
        @return Betriebsstundenzähler [s]
        """

        ret_val = self._counter_operating_seconds
        if self._is_running:
            ret_val += int(time.time() - self._start_time)

        return ret_val

    def get_operating_language(self):
        """!
        Lade die zuletzt verwendete Spracheinstellung der GUI
        @return Zuletzt verwendete Spracheinstellung der GUI
        """
        if self._op_data_path is not None:
            language = self._opData.get('settings', 'language')
        else:
            _logger.debug("Can not load operating data from file 'operating_data.gs1'")
            self._error_handler.set_error("F1000")

        return language

    def get_operating_unit(self):
        """!
        Lade die zuletzt verwendete Temperatureinheit der GUI
        @return Zuletzt verwendete Temperatureinheit der GUI
        """
        if self._op_data_path is not None:
            unit = self._opData.get('settings', 'unit')
        else:
            _logger.debug("Can not load operating data from file 'operating_data.gs1'")
            self._error_handler.set_error("F1000")

        return unit

    def set_operating_language(self, language):
        """!
        Sichere aktuell verwendete Spracheinstellung der GUI
        @param language: aktuell verwendete Spracheinstellung der GUI
        """
        retVal = False
        if self._op_data_path is not None:
            self._opData.set('settings', 'language', language)
            with open(self._op_data_path, 'wb') as opDataFile:
                self._opData.write(opDataFile)
                retVal = True
        else:
            _logger.debug("Can not save process data to file 'operating_data.gs1'")
            self._error_handler.set_error("F1000")
            retVal = False
        return retVal

    def set_operating_unit(self, unit):
        """!
        Sichere aktuell verwendete Temperatureineheit der GUI
        @param language: aktuell verwendete Temperatureineheit der GUI
        """
        retVal = False
        if self._op_data_path is not None:
            self._opData.set('settings', 'unit', unit)
            with open(self._op_data_path, 'wb') as opDataFile:
                self._opData.write(opDataFile)
                retVal = True
        else:
            _logger.debug("Can not save process data to file 'operating_data.gs1'")
            self._error_handler.set_error("F1000")
            retVal = False
        return retVal

    def get_operating_hours(self):
        """!
        Gibt die Betriebsstunden der Heizung zurück.
        @return Betriebsstundenzähler [h]
        """
        minutes = ceil(self.get_operating_seconds() / 60.0)
        hours = int(ceil(minutes / 60.0))
        return hours

    # --- thread running ---
    def process(self):
        """!
        Thread für den Betriebsstundenzähler.
        """

        while not self._stopped.wait(SAVE_EVERY_SECONDS):
            if self._is_running:
                self._store_operating_seconds(self.get_operating_seconds())
